@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
 """
@@ -9,6 +8,7 @@ import sys
 import math
 import heapq
 
+import sqlalchemy as sa
 from collections import OrderedDict
 from sqlalchemy.orm import sessionmaker
 from six import integer_types, string_types
@@ -86,6 +86,7 @@ class SearchEngine(object):
                 download_db_file()
             engine = connect_to_zipcode_db()
             self.zip_klass = Zipcode
+        self.engine = engine
         self.ses = sessionmaker(bind=engine)()
 
     def __enter__(self):  # pragma: no cover
@@ -94,6 +95,11 @@ class SearchEngine(object):
     def __exit__(self, *exc_info):  # pragma: no cover
         self.close()
 
+    def __del__(self):  # pragma: no cover
+        # Cleanup connection if still open
+        if self.ses:
+            self.close()
+
     def close(self):
         """
         close database connection.
@@ -333,6 +339,8 @@ class SearchEngine(object):
         :param median_household_income_lower:
         :param median_household_income_upper:
         :param zipcode_type: str or :class`~uszipcode.model.ZipcodeType` attribute.
+            if None, allows to return any type of zipcode.
+            if specified, only return specified zipcode type.
         :param sort_by: str or :class:`~uszipcode.model.Zipcode` attribute,
             specified which field is used for sorting.
         :param ascending: bool, True means ascending, False means descending.
@@ -421,7 +429,6 @@ class SearchEngine(object):
 
         if zipcode_type is not None:
             filters.append(self.zip_klass.zipcode_type == zipcode_type)
-
         if zipcode is not None:
             filters.append(self.zip_klass.zipcode == str(zipcode))
         if prefix is not None:
@@ -531,13 +538,30 @@ class SearchEngine(object):
             else:
                 return q.all()
 
-    def by_zipcode(self, zipcode):
+    def by_zipcode(self,
+                   zipcode,
+                   zipcode_type=None,
+                   zero_padding=True):
         """
         Search zipcode by exact 5 digits zipcode. No zero padding is needed.
-        """
-        zipcode = str(zipcode).zfill(5)
 
-        res = self.query(zipcode=zipcode, sort_by=None, returns=1)
+        :param zipcode: int or str, the zipcode will be automatically
+            zero padding to 5 digits.
+        :param zipcode_type: str or :class`~uszipcode.model.ZipcodeType` attribute.
+            by default, it returns any zipcode type.
+        :param zero_padding: bool, toggle on and off automatic zero padding.
+        """
+        if zero_padding:
+            zipcode = str(zipcode).zfill(5)
+        else:  # pragma: no cover
+            zipcode = str(zipcode)
+
+        res = self.query(
+            zipcode=zipcode,
+            sort_by=None,
+            returns=1,
+            zipcode_type=zipcode_type,
+        )
         if len(res):
             return res[0]
         else:
@@ -798,3 +822,11 @@ class SearchEngine(object):
             sort_by=sort_by, zipcode_type=zipcode_type,
             ascending=ascending, returns=returns,
         )
+
+    def inspect_raw_data(self, zipcode):
+        sql = "SELECT * FROM {} WHERE zipcode = '{}'".format(
+            self.zip_klass.__tablename__,
+            zipcode,
+        )
+        stmt = sa.text(sql)
+        return dict(self.engine.execute(stmt).fetchone())
